@@ -4,7 +4,7 @@ package YAML::PP::Schema;
 use B;
 use Module::Load qw//;
 
-our $VERSION = '0.011_001'; # TRIAL VERSION
+our $VERSION = '0.011_002'; # TRIAL VERSION
 
 use YAML::PP::Common qw/ YAML_PLAIN_SCALAR_STYLE /;
 
@@ -56,19 +56,44 @@ sub true { return $_[0]->{true} }
 sub false { return $_[0]->{false} }
 sub bool_class { return $_[0]->{bool_class} }
 
+my %LOADED_SCHEMA = (
+    Failsafe => 1,
+    JSON => 1,
+    Core => 1,
+);
 sub load_subschemas {
     my ($self, @schemas) = @_;
-    for my $s (@schemas) {
+    my $i = 0;
+    while ($i < @schemas) {
+        my $item = $schemas[ $i ];
+        $i++;
+        my %options;
+        while ($i < @schemas and $schemas[ $i ] =~ m/^\+(\w+)$/) {
+            my $option = $1;
+            $options{with}->{ $option } = 1;
+            $i++;
+        }
+
         my $class;
-        if ($s =~ m/^\:(.*)/) {
-          $class = "$1";
-          Module::Load::load $class;
-        } else {
-          $class = "YAML::PP::Schema::$s";
+        if ($item =~ m/^\:(.*)/) {
+            $class = "$1";
+            unless ($class =~ m/\A[A-Za-z0-9_:]+\z/) {
+                die "Module name '$class' is invalid";
+            }
+            Module::Load::load $class;
+        }
+        else {
+            $class = "YAML::PP::Schema::$item";
+            unless ($class =~ m/\A[A-Za-z0-9_:]+\z/) {
+                die "Module name '$class' is invalid";
+            }
+            $LOADED_SCHEMA{ $item } ||= Module::Load::load $class;
         }
         my $tags = $class->register(
             schema => $self,
+            options => \%options,
         );
+
     }
 }
 
@@ -77,27 +102,71 @@ sub add_resolver {
     my $tag = $args{tag};
     my $rule = $args{match};
     my $resolvers = $self->resolvers;
-    my ($type, $match, $value) = @$rule;
+    my ($type, @rule) = @$rule;
     my $implicit = $args{implicit};
     $implicit = 1 unless defined $implicit;
-    my @resolvers;
+    my $resolver_list = [];
     if ($tag) {
-        my $res = $resolvers->{tag}->{ $tag } ||= {};
-        push @resolvers, $res;
+        if (ref $tag eq 'Regexp') {
+            my $res = $resolvers->{tags} ||= [];
+            push @$res, [ $tag, {} ];
+            push @$resolver_list, $res->[-1]->[1];
+        }
+        else {
+            my $res = $resolvers->{tag}->{ $tag } ||= {};
+            push @$resolver_list, $res;
+        }
     }
     if ($implicit) {
-        push @resolvers, $resolvers->{value} ||= {};
+        push @$resolver_list, $resolvers->{value} ||= {};
     }
-    for my $res (@resolvers) {
+    for my $res (@$resolver_list) {
         if ($type eq 'equals') {
+            my ($match, $value) = @rule;
             unless (exists $res->{equals}->{ $match }) {
                 $res->{equals}->{ $match } = $value;
             }
             next;
         }
-        if ($type eq 'regex') {
+        elsif ($type eq 'regex') {
+            my ($match, $value) = @rule;
             push @{ $res->{regex} }, [ $match => $value ];
         }
+        elsif ($type eq 'all') {
+            my ($value) = @rule;
+            $res->{all} = $value;
+        }
+    }
+}
+
+sub add_sequence_resolver {
+    my ($self, %args) = @_;
+    return $self->add_collection_resolver(sequence => %args);
+}
+
+sub add_mapping_resolver {
+    my ($self, %args) = @_;
+    return $self->add_collection_resolver(mapping => %args);
+}
+
+sub add_collection_resolver {
+    my ($self, $type, %args) = @_;
+    my $tag = $args{tag};
+    my $implicit = $args{implicit};
+    my $resolvers = $self->resolvers;
+
+    if ($tag and ref $tag eq 'Regexp') {
+        my $res = $resolvers->{ $type }->{tags} ||= [];
+        push @$res, [ $tag, {
+            on_create => $args{on_create},
+            on_data => $args{on_data},
+        } ];
+    }
+    elsif ($tag) {
+        my $res = $resolvers->{ $type }->{tag}->{ $tag } ||= {
+            on_create => $args{on_create},
+            on_data => $args{on_data},
+        };
     }
 }
 
@@ -179,6 +248,15 @@ sub load_scalar {
     my $res;
     if ($tag) {
         $res = $resolvers->{tag}->{ $tag };
+        if (not $res and my $matches = $resolvers->{tags}) {
+            for my $match (@$matches) {
+                my ($re, $rule) = @$match;
+                if ($tag =~ $re) {
+                    $res = $rule;
+                    last;
+                }
+            }
+        }
     }
     else {
         $res = $resolvers->{value};
@@ -205,9 +283,69 @@ sub load_scalar {
             }
         }
     }
+    if (my $catch_all = $res->{all}) {
+        if (ref $catch_all eq 'CODE') {
+            return $catch_all->($constructor, $event);
+        }
+        return $catch_all;
+    }
     return $value;
 }
 
+sub create_sequence {
+    my ($self, $constructor, $event) = @_;
+    my $tag = $event->{tag};
+    my $data = [];
+    my $on_data;
+
+    my $resolvers = $self->resolvers->{sequence};
+    if ($tag) {
+        if (my $matches = $resolvers->{tags}) {
+            for my $match (@$matches) {
+                my ($re, $actions) = @$match;
+                my $on_create = $actions->{on_create};
+                if ($tag =~ $re) {
+                    $on_data = $actions->{on_data};
+                    $on_create and $data = $on_create->($constructor, $event);
+                    return ($data, $on_data);
+                }
+            }
+        }
+    }
+
+    return ($data, $on_data);
+}
+
+sub create_mapping {
+    my ($self, $constructor, $event) = @_;
+    my $tag = $event->{tag};
+    my $data = {};
+    my $on_data;
+
+    my $resolvers = $self->resolvers->{mapping};
+    if ($tag) {
+        if (my $matches = $resolvers->{tags}) {
+            for my $match (@$matches) {
+                my ($re, $actions) = @$match;
+                my $on_create = $actions->{on_create};
+                if ($tag =~ $re) {
+                    $on_data = $actions->{on_data};
+                    $on_create and $data = $on_create->($constructor, $event);
+                    return ($data, $on_data);
+                }
+            }
+        }
+        if (my $equals = $resolvers->{tag}->{ $tag }) {
+            my $on_create = $equals->{on_create};
+            $on_data = $equals->{on_data};
+            $on_create and $data = $on_create->($constructor, $event);
+            return ($data, $on_data);
+        }
+    }
+
+    return ($data, $on_data);
+}
+
 sub bool_jsonpp_true { JSON::PP::true() }
 
 sub bool_booleanpm_true { boolean::true() }
@@ -296,7 +434,7 @@ sub register {
     );
     $schema->add_resolver(
         tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[2]->[0] } ],
+        match => [ all => sub { $_[1]->{value} } ],
         implicit => 0,
     );
 
@@ -453,7 +591,7 @@ sub register {
     ) for (qw/ .nan .NaN .NAN /);
     $schema->add_resolver(
         tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[2]->[0] } ],
+        match => [ all => sub { $_[1]->{value} } ],
         implicit => 0,
     );
 
